@@ -1,5 +1,5 @@
 /*
- * "$Id: escputil.c,v 1.87.8.3 2007/12/29 20:42:25 rlk Exp $"
+ * "$Id: escputil.c,v 1.99 2008/12/21 18:35:43 rlk Exp $"
  *
  *   Printer maintenance utility for EPSON Stylus (R) printers
  *
@@ -279,6 +279,14 @@ initialize_print_cmd(int do_init)
     exit_packet_mode_old(do_init);
 }
 
+static void
+initialize_print_cmd_new(int do_init)
+{
+  bufpos = 0;
+  STP_DEBUG(fprintf(stderr, "Initialize print command (force new)\n"));
+  exit_packet_mode_old(do_init);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -539,7 +547,7 @@ read_from_printer(int fd, char *buf, int bufsize, int quiet)
   int retry = 100;
 #ifdef HAVE_FCNTL_H
   fcntl(fd, F_SETFL,
-	O_NONBLOCK | fcntl(fd, F_GETFL));
+	O_NONBLOCK | (fcntl(fd, F_GETFL) & ~(O_RDONLY|O_WRONLY|O_RDWR)));
 #endif
   memset(buf, 0, bufsize);
 
@@ -784,13 +792,32 @@ set_printer_model(void)
   the_printer_t = NULL;
 }
 
+static int
+test_for_di(const unsigned char *buf)
+{
+  return !(strncmp("di", (const char *)buf, 2) &&
+	   strncmp("@EJL ID", (const char *)buf, 7));
+}
+
+static int
+test_for_st(const unsigned char *buf)
+{
+  return !(strncmp("st", (const char *)buf, 2) &&
+	   strncmp("@BDC ST", (const char *)buf, 7));
+}
+
+static int
+test_for_ii(const unsigned char *buf)
+{
+  return !(strncmp("ii", (const char *)buf, 2) &&
+	   strncmp("@BDC PS", (const char *)buf, 7));
+}
+
 static const stp_printer_t *
 initialize_printer(int quiet, int fail_if_not_found)
 {
   int packet_initialized = 0;
   int fd;
-  int credit;
-  int retry = 4;
   int tries = 0;
   int status;
   int forced_packet_mode = 0;
@@ -847,38 +874,29 @@ initialize_printer(int quiet, int fail_if_not_found)
 
       if (forced_packet_mode || ((buf[3] == status) && (buf[6] == 0x7f)))
 	{
+	  /*
+	   * NX100 looks like it's in D4 mode, but it doesn't really
+	   * respond correctly.  So we force it out of D4 mode and
+	   * then back in to ensure that it's right.  Trying to force
+	   * it into D4 mode alone isn't good enough.
+	   */
+	  initialize_print_cmd_new(0);
+	  (void) SafeWrite(fd, printer_cmd, bufpos);
+	  flushData(fd, (unsigned char) -1);
+	  forced_packet_mode = !init_packet(fd, 1);
 	  STP_DEBUG(fprintf(stderr, "Printer in packet mode....\n"));
 	  packet_initialized = 1;
 	  isnew = 1;
-
-	  credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-	  if (credit < 0)
-	    {
-	      STP_DEBUG(fprintf(stderr, "Cannot get credit\n"));
-	      return NULL;
-	    }
 	  /* request status command */
-	  status = writeData(fd, socket_id, (const unsigned char*)"di\1\0\1",
-			     5, 1);
+	  status =
+	    writeAndReadData(fd, socket_id, (const unsigned char*)"di\1\0\1",
+			     5, 1, (unsigned char *) buf, 1023,
+			     &send_size, &receive_size, &test_for_di);
 	  if (status <= 0)
 	    {
 	      fprintf(stderr, _("\nCannot write to %s: %s\n"),
 		      raw_device, strerror(errno));
-	      return NULL;
-	    }
-	  do
-	    {
-	      status = readData(fd, socket_id, (unsigned char*)buf, 1023);
-	      STP_DEBUG(fprintf(stderr, "readData try %d status %d\n",
-				retry, status));
-	      if (status <= -1 )
-		return NULL;
-	    }
-	  while ( (retry-- != 0) && strncmp("di", (char*)buf, 2) &&
-		  strncmp("@EJL ID", (char*)buf, 7));
-	  if (!retry)
-	    {
-	      STP_DEBUG(fprintf(stderr, "No retries left!\n"));
+	      close(fd);
 	      return NULL;
 	    }
 	}
@@ -887,16 +905,16 @@ initialize_printer(int quiet, int fail_if_not_found)
       if (status > 0)
 	{
 	  pos = strstr((char*)buf, "@EJL ID");
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    pos = strchr(pos, (int) ';');
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    pos = strchr(pos + 1, (int) ';');
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    pos = strchr(pos, (int) ':');
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    {
 	      spos = strchr(pos, (int) ';');
@@ -930,6 +948,7 @@ initialize_printer(int quiet, int fail_if_not_found)
 	      else
 		{
 		  STP_DEBUG(fprintf(stderr, "Can't get response to @EJL ID\n"));
+		  close(fd);
 		  return NULL;
 		}
 	    }
@@ -972,22 +991,23 @@ get_printer(int quiet, int fail_if_not_found)
 
 static const char *colors_new[] =
   {
-    N_("Black"),		/* 0 */
-    N_("Photo Black"),		/* 1 */
-    N_("Unknown"),		/* 2 */
-    N_("Cyan"),			/* 3 */
-    N_("Magenta"),		/* 4 */
-    N_("Yellow"),		/* 5 */
-    N_("Light Cyan"),		/* 6 */
-    N_("Light Magenta"),	/* 7 */
-    N_("Unknown"),		/* 8 */
-    N_("Unknown"),		/* 9 */
-    N_("Light Black"),		/* a */
-    N_("Matte Black"),		/* b */
-    N_("Red"),			/* c */
-    N_("Blue"),			/* d */
-    N_("Gloss Optimizer"),	/* e */
-    N_("Light Light Black"),	/* f */
+    N_("Black"),		/* 00 */
+    N_("Photo Black"),		/* 01 */
+    N_("Unknown"),		/* 02 */
+    N_("Cyan"),			/* 03 */
+    N_("Magenta"),		/* 04 */
+    N_("Yellow"),		/* 05 */
+    N_("Light Cyan"),		/* 06 */
+    N_("Light Magenta"),	/* 07 */
+    N_("Unknown"),		/* 08 */
+    N_("Unknown"),		/* 09 */
+    N_("Light Black"),		/* 0a */
+    N_("Matte Black"),		/* 0b */
+    N_("Red"),			/* 0c */
+    N_("Blue"),			/* 0d */
+    N_("Gloss Optimizer"),	/* 0e */
+    N_("Light Light Black"),	/* 0f */
+    N_("Orange"),		/* 10 */
   };
 static int color_count = sizeof(colors_new) / sizeof(const char *);
 
@@ -1006,7 +1026,7 @@ static const char *aux_colors[] =
     N_("Blue"),			/* a */
     NULL,			/* b */
     NULL,			/* c */
-    NULL,			/* d */
+    N_("Orange"),		/* d */
     NULL,			/* e */
     NULL,			/* f */
   };
@@ -1073,18 +1093,33 @@ print_error(int param)
     case 6:
       printf(_("Error: Paper out\n"));
       break;
+    case 0xc:
+      printf(_("Error: Miscellaneous paper error\n"));
+      break;
+    case 0x10:
+      printf(_("Error: Maintenance cartridge overflow\n"));
+      break;
     case 0x11:
       printf(_("Error: Wait return from the tear-off position\n"));
       break;
     case 0x12:
       printf(_("Error: Double feed error\n"));
       break;
+    case 0x1a:
+      printf(_("Error: Ink cartridge lever released\n\n"));
+      break;
     case 0x1c:
       printf(_("Error: Unrecoverable cutter error\n"));
       break;
     case 0x1d:
       printf(_("Error: Recoverable cutter jam\n"));
       break;
+    case 0x22:
+      printf(_("Error: No maintenance cartridge present\n"));
+      break;
+    case 0x25:
+      printf(_("Error: Rear cover open\n"));
+      break;
     case 0x29:
       printf(_("Error: CD Tray Out\n"));
       break;
@@ -1094,12 +1129,59 @@ print_error(int param)
     case 0x2b:
       printf(_("Error: Tray cover open\n"));
       break;
+    case 0x36:
+      printf(_("Error: Maintenance cartridge cover open\n"));
+      break;
+    case 0x37:
+      printf(_("Error: Front cover open\n"));
+      break;
+    case 0x41:
+      printf(_("Error: Maintenance request\n"));
+      break;
     default:
       printf(_("Error: Unknown (%d)\n"), param);
       break;
     }
 }
 
+static void
+print_warning(int param, const stp_string_list_t *color_list)
+{
+  if (param >= 0x10 && param < 0x20)
+    {
+      param &= 0xf;
+      if (color_list && param < stp_string_list_count(color_list))
+	printf(_("Warning: %s Ink Low\n"),
+	       gettext(stp_string_list_param(color_list, param)->text));
+      else
+	printf(_("Warning: Channel %d Ink Low\n"), param);
+    }
+  else if (param >= 0x50 && param < 0x60)
+    {
+      param &= 0xf;
+      if (color_list && param < stp_string_list_count(color_list))
+	printf(_("Warning: %s Cleaning Disabled\n"),
+	       gettext(stp_string_list_param(color_list, param)->text));
+      else
+	printf(_("Warning: Channel %d Cleaning \n"), param);
+    }
+  else
+    {
+      switch (param)
+	{
+	case 0x20:
+	  printf(_("Warning: Maintenance cartridge near full\n"));
+	  break;
+	case 0x21:
+	  printf(_("Warning: Maintenance request pending\n"));
+	  break;
+	default:
+	  printf(_("Warning: Unknown (%d)\n"), param);
+	  break;
+	}
+    }
+}
+
 static void
 print_self_printing_state(int param)
 {
@@ -1169,6 +1251,9 @@ print_old_ink_levels(const char *ind, stp_string_list_t *color_list)
 static void
 do_old_status(status_cmd_t cmd, const char *buf, const stp_printer_t *printer)
 {
+  if (cmd == CMD_STATUS)
+    printf(_("Printer Name: %s\n"),
+	   printer ? stp_printer_get_long_name(printer) : _("Unknown"));
   do
     {
       const char *ind;
@@ -1189,9 +1274,9 @@ do_old_status(status_cmd_t cmd, const char *buf, const stp_printer_t *printer)
 		{
 		  color_list = stp_string_list_create_copy(desc.bounds.str);
 		  STP_DEBUG(fprintf(stderr,
-				    "Using color list from driver (%d %d)\n",
-				    stp_string_list_count(desc.bounds.str),
-				    stp_string_list_count(color_list)));
+				    "Using color list from driver (%ld %ld)\n",
+				    (long)stp_string_list_count(desc.bounds.str),
+				    (long)stp_string_list_count(color_list)));
 		  stp_parameter_description_destroy(&desc);
 		}
 	    }
@@ -1224,7 +1309,16 @@ do_new_status(status_cmd_t cmd, char *buf, int bytes,
   int i = 0;
   int j;
   const char *ind;
+  const stp_string_list_t *color_list = NULL;
+  stp_parameter_t desc;
+  const stp_vars_t *printvars = stp_printer_get_defaults(printer);
+  stp_describe_parameter(printvars, "ChannelNames", &desc);
+  if (desc.p_type == STP_PARAMETER_TYPE_STRING_LIST)
+    color_list = desc.bounds.str;
   STP_DEBUG(fprintf(stderr, "New format bytes: %d bytes\n", bytes));
+  if (cmd == CMD_STATUS)
+    printf(_("Printer Name: %s\n"),
+	    printer ? stp_printer_get_long_name(printer) : _("Unknown"));
   while (i < bytes)
     {
       unsigned hdr = buf[i];
@@ -1249,7 +1343,7 @@ do_new_status(status_cmd_t cmd, char *buf, int bytes,
 		printf("%18s    %20d\n",
 		       gettext(aux_colors[(int) ind[1]]), ind[2]);
 	      else
-		printf("%8s 0x%2x 0x%2x    %20d\n",
+		printf("%8s 0x%02x 0x%02x    %20d\n",
 		       _("Unknown"), (unsigned char) ind[0],
 		       (unsigned char) ind[1], ind[2]);
 	      ind += param;
@@ -1271,42 +1365,11 @@ do_new_status(status_cmd_t cmd, char *buf, int bytes,
 	      print_self_printing_state(param);
 	      break; 
 	    case 0x4:	/* Warning */
-#if 0
-	      /*
-	       * Bits mean different things on different printers
-	       * Need to figure out how to do this...
-	       * Maybe we don't really need to, since we're also
-	       * printing out ink levels
-	       */
-	      for (j = 0; j < total_param_count; i++)
+	      for (j = 0; j < total_param_count; j++)
 		{
 		  param = (unsigned) buf[i + j + 2];
-		  switch (param)
-		    {
-		    case 0x10:
-		      printf(_("Warning: Black Ink Low\n"));
-		      break;
-		    case 0x11:
-		      printf(_("Warning: Black Ink Low\n"));
-		      break;
-		    case 0x12:
-		      printf(_("Warning: Black Ink Low\n"));
-		      break;
-		    case 0x13:
-		      printf(_("Warning: Black Ink Low\n"));
-		      break;
-		    case 0x14:
-		      printf(_("Warning: Black Ink Low\n"));
-		      break;
-		    case 0x15:
-		      printf(_("Warning: Black Ink Low\n"));
-		      break;
-		    default:
-		      printf(_("Warning: Unknown (%d)\n"), param);
-		      break;
-		    }
+		  print_warning(param, color_list);
 		}
-#endif
 	      break;
 	    case 0x19:	/* Job name */
 	      if (total_param_count > 5)
@@ -1324,6 +1387,7 @@ do_new_status(status_cmd_t cmd, char *buf, int bytes,
 	}
       i += total_param_count + 2;
     }
+  stp_parameter_description_destroy(&desc);
   exit(0);
 }
 
@@ -1332,7 +1396,6 @@ do_status_command_internal(status_cmd_t cmd)
 {
   int fd;
   int status;
-  int credit;
   int retry = 4;
   char buf[1024];
   const stp_printer_t *printer;
@@ -1365,33 +1428,15 @@ do_status_command_internal(status_cmd_t cmd)
 
   if (isnew)
     {
-      credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-      if (credit < 0)
-        {
-          STP_DEBUG(fprintf(stderr, "\nCannot get credit\n"));
-          exit(1);
-        }
       /* request status command */
-      status = writeData(fd, socket_id, (const unsigned char*)"st\1\0\1",
-			 5, 1);
+      status =
+	writeAndReadData(fd, socket_id, (const unsigned char*)"st\1\0\1",
+			 5, 1, (unsigned char *) buf, 1023,
+			 &send_size, &receive_size, &test_for_st);
       if (status <= 0)
 	{
 	  fprintf(stderr, _("\nCannot write to %s: %s\n"), raw_device, strerror(errno));
-	  exit(1);
-	}
-      do
-	{
-	  status = readData(fd, socket_id, (unsigned char*)buf, 1023);
-	  if (status < 0)
-	    {
-	      exit(1);
-	    }
-	  STP_DEBUG(fprintf(stderr, "readData try %d status %d\n", retry, status));
-	} while ((retry-- != 0) && strncmp("st", buf, 2) &&
-		 strncmp("@BDC ST", buf, 7));
-      /* "@BCD ST ST"  found */
-      if (!retry)
-	{
+	  CloseChannel(fd, socket_id);
 	  exit(1);
 	}
       buf[status] = '\0';
@@ -1440,8 +1485,6 @@ do_extended_ink_info(int extended_output)
 {
   int fd;
   int status;
-  int credit;
-  int retry = 4;
   char buf[1024];
   unsigned val, id, id2, year, year2, month, month2;
   unsigned iv[6];
@@ -1478,9 +1521,9 @@ do_extended_ink_info(int extended_output)
       if (printer)
 	{
 	  color_list = stp_string_list_create_copy(desc.bounds.str);
-	  STP_DEBUG(fprintf(stderr, "Using color list from driver (%d %d)\n",
-			    stp_string_list_count(desc.bounds.str),
-			    stp_string_list_count(color_list)));
+	  STP_DEBUG(fprintf(stderr, "Using color list from driver (%ld %ld)\n",
+			    (long)stp_string_list_count(desc.bounds.str),
+			    (long)stp_string_list_count(color_list)));
 	  stp_parameter_description_destroy(&desc);
 	}
       else
@@ -1491,16 +1534,11 @@ do_extended_ink_info(int extended_output)
 	   * message rather than from the ink list.  This gives us a
 	   * last chance to determine the inks
 	   */
-	  credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-	  if (credit < 0)
-	    {
-	      stp_parameter_description_destroy(&desc);
-	      STP_DEBUG(fprintf(stderr, "Cannot get credit\n"));
-	      exit(1);
-	    }
 	  /* request status command */
-	  status = writeData(fd, socket_id, (const unsigned char*)"st\1\0\1",
-			     5, 1);
+	  status =
+	    writeAndReadData(fd, socket_id, (const unsigned char*)"st\1\0\1",
+			     5, 1, (unsigned char *) buf, 1023,
+			     &send_size, &receive_size, &test_for_st);
 	  if (status <= 0)
 	    {
 	      stp_parameter_description_destroy(&desc);
@@ -1508,25 +1546,6 @@ do_extended_ink_info(int extended_output)
 		      raw_device, strerror(errno));
 	      exit(1);
 	    }
-	  do
-	    {
-	      status = readData(fd, socket_id, (unsigned char*)buf, 1023);
-	      if (status < 0)
-		{
-		  stp_parameter_description_destroy(&desc);
-		  exit(1);
-		}
-	      STP_DEBUG(fprintf(stderr, "readData try %d status %d\n",
-				retry, status));
-	    }
-	  while ((retry-- != 0) && strncmp("st", buf, 2) &&
-		 strncmp("@BDC ST", buf, 7));
-	  /* "@BCD ST ST"  found */
-	  if (!retry)
-	    {
-	      stp_parameter_description_destroy(&desc);
-	      exit(1);
-	    }
 	  buf[status] = '\0';
 	  if ( buf[7] == '2' )
 	    {
@@ -1559,26 +1578,16 @@ do_extended_ink_info(int extended_output)
       for (i = 0; i < stp_string_list_count(color_list); i++)
         {
 	  char req[] = "ii\2\0\1\1";
-          credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-	  if (credit < 0)
-	    exit(1);
 	  req[5] = i + 1;
-	  /* request status command */
-	  status = writeData(fd, socket_id, (const unsigned char*)req, 6, 1);
+	  status =
+	    writeAndReadData(fd, socket_id, (const unsigned char*)req,
+			     6, 1, (unsigned char *) buf, 1023,
+			     &send_size, &receive_size, &test_for_ii);
 	  if (status <= 0)
-	    exit(1);
-	  retry = 4;
-	  do
 	    {
-	      status = readData(fd, socket_id, (unsigned char*) buf, 1023);
-	      if (status < 0)
-		{
-		  exit(1);
-		}
-	    } while ((retry-- != 0) && strncmp("ii", buf, 2) &&
-		     strncmp("@BDC PS", buf, 7));
-	  if (!retry) /* couldn't read answer */
-	    exit(1);
+	      CloseChannel(fd, socket_id);
+	      exit(1);
+	    }
 	  ind = strchr(buf, 'I');
 	  if (!ind)
 	    printf("Cannot identify cartridge in slot %d\n", i);
