@@ -1,5 +1,5 @@
 /*
- * "$Id: print-pcl.c,v 1.145.2.1 2007/12/15 20:35:49 rlk Exp $"
+ * "$Id: print-pcl.c,v 1.156 2008/08/06 22:49:05 rlk Exp $"
  *
  *   Print plug-in HP PCL driver for the GIMP.
  *
@@ -37,7 +37,6 @@
 #include <string.h>
 
 /* #define DEBUG */
-/* #define PCL_DEBUG_DISABLE_COMPRESSION */
 /* #define PCL_DEBUG_DISABLE_BLANKLINE_REMOVAL */
 
 /*
@@ -199,6 +198,9 @@ static const pcl_t pcl_media_types[] =
  */
 
 #define PAPERSOURCE_MOD			16
+#define PAPERSOURCE_340_MOD		(PAPERSOURCE_MOD)
+#define PAPERSOURCE_DJ_MOD		(PAPERSOURCE_MOD << 1)
+#define PAPERSOURCE_ADJ_GUIDE		(PAPERSOURCE_MOD << 2)
 
 #define PCL_PAPERSOURCE_STANDARD	0	/* Don't output code */
 #define PCL_PAPERSOURCE_MANUAL		2
@@ -209,20 +211,24 @@ static const pcl_t pcl_media_types[] =
 #define PCL_PAPERSOURCE_LJ_TRAY3	4
 #define PCL_PAPERSOURCE_LJ_TRAY4	5
 #define PCL_PAPERSOURCE_LJ_TRAY1	8
+#define PCL_PAPERSOURCE_LJ_TRAY1_ADJ	(PCL_PAPERSOURCE_LJ_TRAY1 + PAPERSOURCE_ADJ_GUIDE)
+#define PCL_PAPERSOURCE_LJ_TRAY2_ADJ	(PCL_PAPERSOURCE_LJ_TRAY2 + PAPERSOURCE_ADJ_GUIDE)
+#define PCL_PAPERSOURCE_LJ_TRAY3_ADJ	(PCL_PAPERSOURCE_LJ_TRAY3 + PAPERSOURCE_ADJ_GUIDE)
+#define PCL_PAPERSOURCE_LJ_TRAY4_ADJ	(PCL_PAPERSOURCE_LJ_TRAY4 + PAPERSOURCE_ADJ_GUIDE)
 
 /* Deskjet 340 types */
-#define PCL_PAPERSOURCE_340_PCSF	1 + PAPERSOURCE_MOD
+#define PCL_PAPERSOURCE_340_PCSF	(1 + PAPERSOURCE_340_MOD)
 						/* Portable sheet feeder for 340 */
-#define PCL_PAPERSOURCE_340_DCSF	4 + PAPERSOURCE_MOD
+#define PCL_PAPERSOURCE_340_DCSF	(4 + PAPERSOURCE_340_MOD)
 						/* Desktop sheet feeder for 340 */
 
 /* Other Deskjet types */
-#define PCL_PAPERSOURCE_DJ_TRAY		1 + PAPERSOURCE_MOD + PAPERSOURCE_MOD
-#define PCL_PAPERSOURCE_DJ_TRAY2	4 + PAPERSOURCE_MOD + PAPERSOURCE_MOD
+#define PCL_PAPERSOURCE_DJ_TRAY		(1 + PAPERSOURCE_DJ_MOD)
+#define PCL_PAPERSOURCE_DJ_TRAY2	(4 + PAPERSOURCE_DJ_MOD)
 						/* Tray 2 for 2500 */
-#define PCL_PAPERSOURCE_DJ_OPTIONAL	5 + PAPERSOURCE_MOD + PAPERSOURCE_MOD
+#define PCL_PAPERSOURCE_DJ_OPTIONAL	(5 + PAPERSOURCE_DJ_MOD)
 						/* Optional source for 2500 */
-#define PCL_PAPERSOURCE_DJ_AUTO		7 + PAPERSOURCE_MOD + PAPERSOURCE_MOD
+#define PCL_PAPERSOURCE_DJ_AUTO		(7 + PAPERSOURCE_DJ_MOD)
 						/* Autoselect for 2500 */
 
 static const pcl_t pcl_media_sources[] =
@@ -230,9 +236,13 @@ static const pcl_t pcl_media_sources[] =
     { "Standard", N_ ("Standard"), PCL_PAPERSOURCE_STANDARD},
     { "Manual", N_ ("Manual"), PCL_PAPERSOURCE_MANUAL},
 /*  {"Envelope", PCL_PAPERSOURCE_ENVELOPE}, */
+    { "MultiPurposeAdj", N_ ("Tray 1 - Movable Guides"), PCL_PAPERSOURCE_LJ_TRAY1_ADJ},
     { "MultiPurpose", N_ ("Tray 1"), PCL_PAPERSOURCE_LJ_TRAY1},
+    { "UpperAdj", N_ ("Tray 2 - Movable Guides"), PCL_PAPERSOURCE_LJ_TRAY2_ADJ},
     { "Upper", N_ ("Tray 2"), PCL_PAPERSOURCE_LJ_TRAY2},
+    { "LowerAdj", N_ ("Tray 3 - Movable Guides"), PCL_PAPERSOURCE_LJ_TRAY3_ADJ},
     { "Lower", N_ ("Tray 3"), PCL_PAPERSOURCE_LJ_TRAY3},
+    { "LargeCapacityAdj", N_ ("Tray 4 - Movable Guides"), PCL_PAPERSOURCE_LJ_TRAY4_ADJ},
     { "LargeCapacity", N_ ("Tray 4"), PCL_PAPERSOURCE_LJ_TRAY4},
     { "Portable", N_ ("Portable Sheet Feeder"), PCL_PAPERSOURCE_340_PCSF},
     { "Desktop", N_ ("Desktop Sheet Feeder"), PCL_PAPERSOURCE_340_DCSF},
@@ -263,26 +273,16 @@ static const pcl_t pcl_resolutions[] =
 };
 #define NUM_RESOLUTIONS		(sizeof(pcl_resolutions) / sizeof (pcl_t))
 
-static void
-pcl_describe_resolution(const stp_vars_t *v, int *x, int *y)
+static const pcl_t pcl_qualities[] =
 {
-  int i;
-  const char *resolution = stp_get_string_parameter(v, "Resolution");
-  if (resolution)
-    {
-      for (i = 0; i < NUM_RESOLUTIONS; i++)
-	{
-	  if (!strcmp(resolution, pcl_resolutions[i].pcl_name))
-	    {
-	      *x = pcl_resolutions[i].p0;
-	      *y = pcl_resolutions[i].p1;
-	      return;
-	    }
-	}
-    }
-  *x = -1;
-  *y = -1;
-}
+    { "Draft", N_("Draft"), PCL_RES_150_150, 150, 150},
+    { "Standard", N_("Standard"), PCL_RES_300_300, 300, 300},
+    { "High", N_("High"), PCL_RES_600_600, 600, 600},
+    { "High", N_("Standard"), PCL_RES_600_300, 600, 300},
+    { "Photo", N_("Photo"), PCL_RES_1200_600, 1200, 600},
+    { "Photo", N_("Photo"), PCL_RES_2400_600, 2400, 600},
+};
+#define NUM_QUALITIES		(sizeof(pcl_qualities) / sizeof (pcl_t))
 
 typedef struct {
   int top_margin;
@@ -611,9 +611,13 @@ static const short laserjet_papersources[] =
 {
   PCL_PAPERSOURCE_STANDARD,
   PCL_PAPERSOURCE_MANUAL,
+  PCL_PAPERSOURCE_LJ_TRAY1_ADJ,
   PCL_PAPERSOURCE_LJ_TRAY1,
+  PCL_PAPERSOURCE_LJ_TRAY2_ADJ,
   PCL_PAPERSOURCE_LJ_TRAY2,
+  PCL_PAPERSOURCE_LJ_TRAY3_ADJ,
   PCL_PAPERSOURCE_LJ_TRAY3,
+  PCL_PAPERSOURCE_LJ_TRAY4_ADJ,
   PCL_PAPERSOURCE_LJ_TRAY4,
   -1,
 };
@@ -1034,6 +1038,19 @@ static const pcl_cap_t pcl_model_capabilities[] =
     emptylist,
     laserjet_papersources,
   },
+  /* PCL-4 with large paper */
+  { 23,
+    13 * 72, 19 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300,
+    {12, 12, 18, 18},
+    {12, 12, 10, 10},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ,
+    ljbig_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
   /* LaserJet III series */
   { 3,
     17 * 72 / 2, 14 * 72,
@@ -1047,6 +1064,19 @@ static const pcl_cap_t pcl_model_capabilities[] =
     emptylist,
     laserjet_papersources,
   },
+  /* LaserJet III series */
+  { 31,
+    17 * 72 / 2, 14 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300,
+    {12, 12, 18, 18},
+    {12, 12, 10, 10},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE,
+    ljsmall_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
   /* Some laser printers don't have expanded A4 margins */
   { 32,
     17 * 72 / 2, 14 * 72,
@@ -1060,6 +1090,19 @@ static const pcl_cap_t pcl_model_capabilities[] =
     emptylist,
     laserjet_papersources,
   },
+  /* PCL-5 with large paper */
+  { 33,
+    13 * 72, 19 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300,
+    {12, 12, 18, 18},
+    {12, 12, 10, 10},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE,
+    ljbig_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
   /* LaserJet 4L */
   { 4,
     17 * 72 / 2, 14 * 72,
@@ -1114,6 +1157,20 @@ static const pcl_cap_t pcl_model_capabilities[] =
     emptylist,
     laserjet_papersources,
   },
+  /* PCL-5c/5e/6/XL with large paper */
+  { 61,
+    13 * 72, 19 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300 | PCL_RES_600_600,
+    {12, 12, 18, 18},
+    {12, 12, 10, 10},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_NEW_ERG | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE |
+      PCL_PRINTER_DUPLEX,
+    ljbig_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
   /* Some laser printers don't have expanded A4 margins */
   { 62,
     17 * 72 / 2, 14 * 72,
@@ -1196,49 +1253,55 @@ static const stp_parameter_t the_parameters[] =
     "PageSize", N_("Page Size"), N_("Basic Printer Setup"),
     N_("Size of the paper being printed to"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "MediaType", N_("Media Type"), N_("Basic Printer Setup"),
     N_("Type of media (plain paper, photo paper, etc.)"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "InputSlot", N_("Media Source"), N_("Basic Printer Setup"),
     N_("Source (input slot) of the media"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
+  },
+  {
+    "Quality", N_("Print Quality"), N_("Basic Output Adjustment"),
+    N_("Print Quality"),
+    STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 0, 0
   },
   {
     "Resolution", N_("Resolution"), N_("Basic Printer Setup"),
-    N_("Resolution and quality of the print"),
+    N_("Resolution of the print"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "InkType", N_("Ink Type"), N_("Advanced Printer Setup"),
     N_("Type of ink in the printer"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "InkChannels", N_("Ink Channels"), N_("Advanced Printer Functionality"),
     N_("Ink Channels"),
     STP_PARAMETER_TYPE_INT, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_INTERNAL, 0, 0, -1, 0, 0
+    STP_PARAMETER_LEVEL_INTERNAL, 0, 0, STP_CHANNEL_NONE, 0, 0
   },
   {
     "PrintingMode", N_("Printing Mode"), N_("Core Parameter"),
     N_("Printing Output Mode"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "Duplex", N_("Double-Sided Printing"), N_("Basic Printer Setup"),
     N_("Duplex/Tumble Setting"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
 };
 
@@ -1290,18 +1353,18 @@ static const float_param_t float_parameters[] =
   },
   {
     {
-      "LightCyanTransition", N_("Light Cyan Transition"), N_("Advanced Ink Adjustment"),
+      "LightCyanTrans", N_("Light Cyan Transition"), N_("Advanced Ink Adjustment"),
       N_("Light Cyan Transition"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
-      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, -1, 1, 0
+      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, STP_CHANNEL_NONE, 1, 0
     }, 0.0, 5.0, 1.0, 1
   },
   {
     {
-      "LightMagentaTransition", N_("Light Magenta Transition"), N_("Advanced Ink Adjustment"),
+      "LightMagentaTrans", N_("Light Magenta Transition"), N_("Advanced Ink Adjustment"),
       N_("Light Magenta Transition"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
-      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, -1, 1, 0
+      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, STP_CHANNEL_NONE, 1, 0
     }, 0.0, 5.0, 1.0, 1
   },
 };
@@ -1423,6 +1486,51 @@ pcl_get_model_capabilities(int model)	/* I: Model */
   return &(pcl_model_capabilities[0]);
 }
 
+/*
+ * Determine the current resolution based on quality and resolution settings
+ */
+
+static void
+pcl_describe_resolution(const stp_vars_t *v, int *x, int *y)
+{
+  int i;
+  int model = stp_get_model_id(v);
+  const char *resolution = stp_get_string_parameter(v, "Resolution");
+  const char *quality;
+  const pcl_cap_t *caps = NULL;
+  if (resolution)
+    {
+      for (i = 0; i < NUM_RESOLUTIONS; i++)
+	{
+	  if (!strcmp(resolution, pcl_resolutions[i].pcl_name))
+	    {
+	      *x = pcl_resolutions[i].p0;
+	      *y = pcl_resolutions[i].p1;
+	      return;
+	    }
+	}
+    }
+  quality = stp_get_string_parameter(v, "Quality");
+  caps = pcl_get_model_capabilities(model);
+  if (quality && strcmp(quality, "None") == 0)
+    quality = "Standard";
+  if (quality)
+    {
+      for (i = 0; i < NUM_QUALITIES; i++)
+	{
+	  if ((caps->resolutions & pcl_qualities[i].pcl_code) &&
+	      !strcmp(quality, pcl_qualities[i].pcl_name))
+	    {
+	      *x = pcl_qualities[i].p0;
+	      *y = pcl_qualities[i].p1;
+	      return;
+	    }
+	}
+    }
+  *x = -1;
+  *y = -1;
+}
+
 /*
  * Convert Media size name into PCL media code for printer
  */
@@ -1594,6 +1702,14 @@ pcl_parameters(const stp_vars_t *v, const char *name,
   stp_deprintf(STP_DBG_PCL, "Resolutions: %d\n", caps->resolutions);
   stp_deprintf(STP_DBG_PCL, "ColorType = %d, PrinterType = %d\n", caps->color_type, caps->stp_printer_type);
 
+  for (i = 0; i < the_parameter_count; i++)
+    if (strcmp(name, the_parameters[i].name) == 0)
+      {
+	stp_fill_parameter_settings(description, &(the_parameters[i]));
+	break;
+      }
+  description->deflt.str = NULL;
+
   for (i = 0; i < float_parameter_count; i++)
     if (strcmp(name, float_parameters[i].param.name) == 0)
       {
@@ -1602,15 +1718,8 @@ pcl_parameters(const stp_vars_t *v, const char *name,
 	description->deflt.dbl = float_parameters[i].defval;
 	description->bounds.dbl.upper = float_parameters[i].max;
 	description->bounds.dbl.lower = float_parameters[i].min;
-      }
-
-  for (i = 0; i < the_parameter_count; i++)
-    if (strcmp(name, the_parameters[i].name) == 0)
-      {
-	stp_fill_parameter_settings(description, &(the_parameters[i]));
 	break;
       }
-  description->deflt.str = NULL;
 
   if (strcmp(name, "PageSize") == 0)
     {
@@ -1667,15 +1776,11 @@ pcl_parameters(const stp_vars_t *v, const char *name,
   else if (strcmp(name, "Resolution") == 0)
   {
     description->bounds.str = stp_string_list_create();
-    description->deflt.str = NULL;
+    stp_string_list_add_string(description->bounds.str, "None", _("Default"));
+    description->deflt.str = "None";
     for (i = 0; i < NUM_RESOLUTIONS; i++)
       if (caps->resolutions & pcl_resolutions[i].pcl_code)
 	{
-	  if (pcl_resolutions[i].pcl_code >= PCL_RES_300_300 &&
-	      description->deflt.str == NULL)
-	    description->deflt.str =
-	      pcl_val_to_string(pcl_resolutions[i].pcl_code,
-				pcl_resolutions, NUM_RESOLUTIONS);
 	  stp_string_list_add_string
 	    (description->bounds.str,
 	     pcl_val_to_string(pcl_resolutions[i].pcl_code,
@@ -1683,8 +1788,31 @@ pcl_parameters(const stp_vars_t *v, const char *name,
 	     pcl_val_to_text(pcl_resolutions[i].pcl_code,
 			     pcl_resolutions, NUM_RESOLUTIONS));
 	}
-    if (description->deflt.str == NULL)
-      stp_erprintf("No default resolution set!\n");
+  }
+  else if (strcmp(name, "Quality") == 0)
+  {
+    int has_standard_quality = 0;
+    description->bounds.str = stp_string_list_create();
+    stp_string_list_add_string(description->bounds.str, "None",
+			       _("Manual Control"));
+    for (i = 0; i < NUM_QUALITIES; i++)
+      if (caps->resolutions & pcl_qualities[i].pcl_code)
+	{
+	  const char *qual =
+	    pcl_val_to_string(pcl_qualities[i].pcl_code,
+			      pcl_qualities, NUM_QUALITIES);
+	  if (! stp_string_list_is_present(description->bounds.str, qual))
+	    stp_string_list_add_string
+	      (description->bounds.str, qual,
+	       pcl_val_to_text(pcl_qualities[i].pcl_code,
+			       pcl_qualities, NUM_QUALITIES));
+	  if (strcmp(qual, "Standard") == 0)
+	    has_standard_quality = 1;
+	}
+    if (has_standard_quality)
+      description->deflt.str = "Standard";
+    else
+      description->deflt.str = "None";
   }
   else if (strcmp(name, "InkType") == 0)
   {
@@ -1747,8 +1875,8 @@ pcl_parameters(const stp_vars_t *v, const char *name,
       else
 	description->is_active = 0;
     }
-  else if (strcmp(name, "LightCyanTransition") == 0 ||
-	   strcmp(name, "LightMagentaTransition") == 0)
+  else if (strcmp(name, "LightCyanTrans") == 0 ||
+	   strcmp(name, "LightMagentaTrans") == 0)
     {
       if (caps->color_type & PCL_COLOR_CMYKcm &&
 	  stp_check_string_parameter(v, "PrintingMode", STP_PARAMETER_DEFAULTED) &&
@@ -2032,11 +2160,9 @@ get_double_param(stp_vars_t *v, const char *param)
 static int
 pcl_do_print(stp_vars_t *v, stp_image_t *image)
 {
-  int i;
   pcl_privdata_t privdata;
   int		status = 1;
   int		model = stp_get_model_id(v);
-  const char	*resolution = stp_get_string_parameter(v, "Resolution");
   const char	*media_size = stp_get_string_parameter(v, "PageSize");
   const char	*media_type = stp_get_string_parameter(v, "MediaType");
   const char	*media_source = stp_get_string_parameter(v, "InputSlot");
@@ -2079,6 +2205,8 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
   const stp_papersize_t *pp;
   int		the_top_margin,	/* Corrected top margin */
 		the_left_margin;	/* Corrected left margin */
+  int		manual_feed_left_adjust = 0;
+  int		extra_left_margin = 0;
   stp_curve_t   *lum_adjustment;
   stp_curve_t   *hue_adjustment;
   double density;
@@ -2105,24 +2233,14 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
   * Figure out the output resolution...
   */
 
-  xdpi = 0;
-  ydpi = 0;
-  if (resolution)
-    {
-      for (i = 0; i < NUM_RESOLUTIONS; i++)
-	{
-	  if (!strcmp(resolution, pcl_resolutions[i].pcl_name))
-	    {
-	      xdpi = pcl_resolutions[i].p0;
-	      ydpi = pcl_resolutions[i].p1;
-	      break;
-	    }
-	}
-    }
+  pcl_describe_resolution(v, &xdpi, &ydpi);
 
   stp_deprintf(STP_DBG_PCL,"pcl: resolution=%dx%d\n",xdpi,ydpi);
-  if (xdpi == 0 || ydpi == 0)
-    return 0;
+  if (xdpi <= 0 || ydpi <= 0)
+    {
+      stp_eprintf(v, "No resolution found; cannot print.\n");
+      return 0;
+    }
 
  /*
   * Choose the correct color conversion function...
@@ -2264,6 +2382,12 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
 
 /* Correct the value by taking the modulus */
 
+	  if ((pcl_media_source & PAPERSOURCE_ADJ_GUIDE) ==
+	      PAPERSOURCE_ADJ_GUIDE)
+	    {
+	      manual_feed_left_adjust = 1;
+	      stp_deprintf(STP_DBG_PCL, "Adjusting left margin for manual feed.\n");
+	    }
           pcl_media_source = pcl_media_source % PAPERSOURCE_MOD;
           stp_zprintf(v, "\033&l%dH", pcl_media_source);
         }
@@ -2422,13 +2546,12 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
     }
   }
 
-#ifndef PCL_DEBUG_DISABLE_COMPRESSION
-  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF)
+  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF &&
+      !(stp_get_debug_level() & STP_DBG_NO_COMPRESSION))
   {
     stp_puts("\033*b2M", v);			/* Mode 2 (TIFF) */
   }
   else
-#endif
   {
     stp_puts("\033*b0M", v);			/* Mode 0 (no compression) */
   }
@@ -2454,8 +2577,26 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
   stp_deprintf(STP_DBG_PCL, "left %d margin %d top %d margin %d width %d height %d\n",
 	  left, the_left_margin, top, the_top_margin, out_width, out_height);
 
+  if (manual_feed_left_adjust)
+    {
+      unsigned wdelta = caps->custom_max_width - stp_get_page_width(v);
+      if (wdelta > 0)
+	{
+	  /*
+	   * Why 3?  I would expect it would be 2 here, but it appears
+	   * that at least one printer (LJ 1022) actually partially
+	   * adjusts the margin itself.  Adjusting the left margin by 1/3
+	   * of the difference between the maximum width and the actual
+	   * width experimentally yields correct results -- rlk 20081014
+	   */
+	  stp_deprintf(STP_DBG_PCL,
+		       "  Adjusting manual feed left margin by %d\n", wdelta / 3);
+	  extra_left_margin += wdelta / 3;
+	}
+    }
+
   if (!privdata.do_cretb) {
-    stp_zprintf(v, "\033&a%dH", 10 * left);		/* Set left raster position */
+    stp_zprintf(v, "\033&a%dH", 10 * (left + extra_left_margin));		/* Set left raster position */
     stp_zprintf(v, "\033&a%dV", 10 * (top + the_top_margin));
 				/* Set top raster position */
   }
@@ -2466,7 +2607,7 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
     {
       /* Move to top left of printed area */
       stp_zprintf(v, "\033*p%dY", (top + the_top_margin)*4); /* Measured in dots. */
-      stp_zprintf(v, "\033*p%dX", left*4);
+      stp_zprintf(v, "\033*p%dX", (left + extra_left_margin)*4);
     }
   stp_puts("\033*r1A", v); 			/* Start GFX */
 
@@ -2521,14 +2662,13 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
 
 /* Allocate buffer for pcl_mode2 tiff compression */
 
-#ifndef PCL_DEBUG_DISABLE_COMPRESSION
-  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF)
+  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF &&
+      !(stp_get_debug_level() & STP_DBG_NO_COMPRESSION))
   {
     privdata.comp_buf = stp_malloc((privdata.height + 128 + 7) * 129 / 128);
     privdata.writefunc = pcl_mode2;
   }
   else
-#endif
   {
     privdata.comp_buf = NULL;
     privdata.writefunc = pcl_mode0;
@@ -2628,12 +2768,12 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
   if (lcyan)
     stp_channel_set_density_adjustment
       (v, STP_ECOLOR_C, 1, (get_double_param(v, "CyanDensity") *
-			get_double_param(v, "LightCyanTransition") *
+			get_double_param(v, "LightCyanTrans") *
 			get_double_param(v, "Density")));
   if (lmagenta)
     stp_channel_set_density_adjustment
       (v, STP_ECOLOR_M, 1, (get_double_param(v, "MagentaDensity") *
-			get_double_param(v, "LightMagentaTransition") *
+			get_double_param(v, "LightMagentaTrans") *
 			get_double_param(v, "Density")));
 
 
